import wx
import wx.stc
import wx.html
import sys

class PrefDialog(wx.Dialog):
    def __init__(self,parent):
        wx.Dialog.__init__(self,parent,title='Preferences')
        
        sizer=wx.BoxSizer(wx.VERTICAL)
        
        spacer=(5,5)
        
        sizer.Add(spacer)
        
        self.bg=wx.ColourPickerCtrl(self,-1,style=wx.CLRP_SHOW_LABEL,col=parent.editorBackground)
        self.fg=wx.ColourPickerCtrl(self,-1,style=wx.CLRP_SHOW_LABEL,col=parent.editorForeground)
        self.lbg=wx.ColourPickerCtrl(self,-1,style=wx.CLRP_SHOW_LABEL,col=parent.editoLineBackground)
        
        pref_sizer=wx.FlexGridSizer(3, 5, 5, 5)
        
        pref_sizer.Add(spacer)
        pref_sizer.Add(wx.StaticText(self, -1, "Background:"), 1, wx.ALIGN_CENTER_VERTICAL)
        pref_sizer.Add(spacer)
        pref_sizer.Add(self.bg, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        pref_sizer.Add(spacer)
        
        pref_sizer.Add(spacer)
        pref_sizer.Add(wx.StaticText(self, -1, "Foreground:"), 1, wx.ALIGN_CENTER_VERTICAL)
        pref_sizer.Add(spacer)
        pref_sizer.Add(self.fg, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        pref_sizer.Add(spacer)
        
        pref_sizer.Add(spacer)
        pref_sizer.Add(wx.StaticText(self, -1, "Current Line:"), 1, wx.ALIGN_CENTER_VERTICAL)
        pref_sizer.Add(spacer)
        pref_sizer.Add(self.lbg, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        pref_sizer.Add(spacer)
        
        sizer.Add(pref_sizer, 1, wx.GROW|wx.EXPAND)
        
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)
        
        cancel=wx.Button(self,wx.ID_CANCEL,"Cancel")
        ok=wx.Button(self,wx.ID_OK,"Ok")
        ok.SetDefault()
        
        
        
        button_sizer.Add(cancel)
        button_sizer.Add(spacer)
        button_sizer.Add(ok)
        
        sizer.Add(spacer)
        sizer.Add(spacer)
        sizer.Add(button_sizer, 0, wx.CENTER)
        sizer.Add(spacer)
        
        self.SetSizer(sizer)
        
        self.SetAutoLayout(True)
        self.Layout()
        
        self.SetSize(self.GetBestSize())

class MainFrame(wx.Frame):
    def __init__(self,parent):
        wx.Frame.__init__(self,parent,title='5KEdit',style=wx.MAXIMIZE)
        
        self.menubar=wx.MenuBar()
        file_menu=wx.Menu()

        self.Bind(wx.EVT_MENU, self.OnNew, file_menu.Append(-1,"New\tCtrl-N"))
        self.Bind(wx.EVT_MENU, self.OnOpen, file_menu.Append(-1,"Open...\tCtrl-O"))
        file_menu.AppendSeparator()
        self.Bind(wx.EVT_MENU, self.OnSave, file_menu.Append(-1,"Save\tCtrl-S"))
        self.Bind(wx.EVT_MENU, self.OnSaveAs, file_menu.Append(-1,"Save As...\tShift-Ctrl-S"))
        file_menu.AppendSeparator()
        self.Bind(wx.EVT_MENU, self.OnPreferences, file_menu.Append(-1,"Preferences...\tCtrl-K"))
        file_menu.AppendSeparator()
        self.Bind(wx.EVT_MENU, self.OnQuit, file_menu.Append(-1,"Quit 5KEdit\tCtrl-Q"))
        
        edit_menu=wx.Menu()
        self.Bind(wx.EVT_MENU, self.OnUndo, edit_menu.Append(-1,"Undo\tCtrl-Z"))
        self.Bind(wx.EVT_MENU, self.OnRedo, edit_menu.Append(-1,"Redo\tShift-Ctrl-Z"))
        edit_menu.AppendSeparator()
        self.Bind(wx.EVT_MENU, self.OnCut,   edit_menu.Append(-1,"Cut\tCtrl-X"))
        self.Bind(wx.EVT_MENU, self.OnCopy,  edit_menu.Append(-1,"Copy\tCtrl-C"))
        self.Bind(wx.EVT_MENU, self.OnPaste, edit_menu.Append(-1,"Paste\tCtrl-P"))
        edit_menu.AppendSeparator()
        self.Bind(wx.EVT_MENU, self.OnSelectAll, edit_menu.Append(-1,"Select All\tCtrl-A"))
        edit_menu.AppendSeparator()
        self.Bind(wx.EVT_MENU, self.OnFind, edit_menu.Append(-1,"Find...\tCtrl-F"))
        
        view_menu=wx.Menu()
        self.Bind(wx.EVT_MENU, self.OnFontBigger,  view_menu.Append(-1,"Make Text Bigger\tCtrl-B"))
        self.Bind(wx.EVT_MENU, self.OnFontSmaller, view_menu.Append(-1,"Make Text Smaller\tShift-Ctrl-B"))
        
        text_menu=wx.Menu()
        self.Bind(wx.EVT_MENU, self.OnWordCount,  text_menu.Append(-1,"Word Count\tShift-Ctrl-W"))
        
        nav_menu=wx.Menu()
        self.Bind(wx.EVT_MENU, self.OnPageUp, nav_menu.Append(-1,"Page Up\tCtrl-Up"))
        self.Bind(wx.EVT_MENU, self.OnPageDown, nav_menu.Append(-1,"Page Down\tCtrl-Down"))
        nav_menu.AppendSeparator()
        self.Bind(wx.EVT_MENU, self.OnGotoLine, nav_menu.Append(-1,"Goto Line...\tCtrl-G"))
        
        help_menu=wx.Menu()
        self.Bind(wx.EVT_MENU, self.OnList, help_menu.Append(-1,"List Commands\tCtrl-L"))
        self.Bind(wx.EVT_MENU, self.OnInfo, help_menu.Append(-1,"About\tCtrl-I"))
        
        self.menubar.Append(file_menu, "&File")
        self.menubar.Append(edit_menu, "&Edit")
        self.menubar.Append(view_menu, "&View")
        self.menubar.Append(text_menu, "&Text")
        self.menubar.Append(nav_menu, "&Navigation")
        self.menubar.Append(help_menu, "&Help")
        
        self.SetMenuBar(self.menubar)
        
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        
        sizer=wx.BoxSizer(wx.VERTICAL)
        
        self.text=wx.stc.StyledTextCtrl(self,style=wx.TE_MULTILINE|wx.NO_BORDER|wx.WANTS_CHARS)
        self.text.SetCaretLineVisible(True)
        self.text.SetCaretWidth(3)
        self.text.SetUseAntiAliasing(True)
        
        style=self.text.GetStyleAt(0)
        self.text.StyleSetEOLFilled(style,True)
        self.text.SetMargins(0,0)
        self.text.SetMarginWidth(1,0)
        self.text.SetWrapMode(wx.stc.STC_WRAP_WORD)
        
        sizer.Add(self.text, 1, flag=wx.EXPAND)
        
        self.status=wx.StaticText(self,-1,' ')
        
        bottom_sizer=wx.BoxSizer(wx.HORIZONTAL)
        
        bottom_sizer.Add(self.status, 1, flag=wx.EXPAND)
        
        self.help=wx.StaticText(self,-1,self._ConvertAccel('List Commands\tCtrl-L'))
        
        bottom_sizer.Add(self.help, 0, flag=wx.EXPAND)
        
        sizer.Add(bottom_sizer, 0, flag=wx.GROW)
        
        self._file_name=None
        self._find_text=None
        
        config=wx.ConfigBase.Get()
        
        # default is green on black with light-grey bg current line
        self.SetEditorForeground(config.Read('5KEditFG','#00FF00'))
        self.SetEditorBackground(config.Read('5KEditBG','#000000'))
        self.SetEditorLineBackground(config.Read('5KEditLBG','#111111'))
        self.SetFontSize(self.GetFontSize())
        
        self.UpdateStatus()
        
        self.Bind(wx.stc.EVT_STC_MODIFIED, self.UpdateStatus)
        self.Bind(wx.stc.EVT_STC_SAVEPOINTLEFT, self.UpdateStatus)
        self.Bind(wx.stc.EVT_STC_SAVEPOINTREACHED, self.UpdateStatus)
        
        self.SetAutoLayout(True)
        self.SetSizer(sizer)
        self.Layout()
    
    def GetFontSize(self):
        config=wx.ConfigBase.Get()
        return config.ReadInt('5KEditFontSize',24)
    
    def SetFontSize(self, size):
        config=wx.ConfigBase.Get()
        config.WriteInt('5KEditFontSize', size)
        font=wx.Font(size,wx.FONTFAMILY_TELETYPE,wx.FONTSTYLE_NORMAL,wx.FONTWEIGHT_NORMAL)
        self.text.SetFont(font)
        self.status.SetFont(font)
        self.help.SetFont(font)
        style=self.text.GetStyleAt(0)
        self.text.StyleSetFont(style,font)
    
    def _SetColorPref(self,key,color):
        config=wx.ConfigBase.Get()
        try:
            config.Write(key,color)
        except TypeError:
            config.Write(key,color.GetAsString(wx.C2S_HTML_SYNTAX))
    
    def SetEditorForeground(self,color):
        self._SetColorPref('5KEditFG',color)
        self.editorForeground=color
        self.text.SetForegroundColour(color)
        self.text.SetCaretForeground(color)
        self.text.SetSelBackground(True, color)
        style=self.text.GetStyleAt(0)
        self.text.StyleSetForeground(style,color)
        self.status.SetForegroundColour(color)
        self.help.SetForegroundColour(color)
    
    def SetEditorBackground(self,color):
        self._SetColorPref('5KEditBG',color)
        self.editorBackground=color
        self.SetBackgroundColour(color)
        self.text.SetBackgroundColour(color)
        self.text.SetSelForeground(True, color)
        self.text.StyleSetBackground(wx.stc.STC_STYLE_DEFAULT,color)
        style=self.text.GetStyleAt(0)
        self.text.StyleSetBackground(style,color)
        self.status.SetBackgroundColour(color)
        self.help.SetBackgroundColour(color)
    
    def SetEditorLineBackground(self,color):
        self._SetColorPref('5KEditLBG',color)
        self.editoLineBackground=color
        self.text.SetCaretLineBackground(color)
    
    def UpdateStatus(self,event=None):
        status='<Untitled>'
        if self._file_name:
            status=self._file_name
        if self.text.GetModify():
            status += '*'
        self.status.SetLabel(status)
        self.Layout()
    
    def Load(self,text_file):
        self._file_name=text_file
        self.text.LoadFile(text_file)
        #self.text.Repaint()
    
    def OnNew(self, event):
        if self.CheckModified():
            self._file_name=None
            self.text.SetText('')
            self.text.EmptyUndoBuffer()
            self.text.SetSavePoint()
    
    def OnOpen(self, event):
        if self.CheckModified():
            file_name=self.ShowOpenDialog()
            if file_name:
                self.Load(file_name)
    
    def OnSave(self, event=None):
        if self._file_name is None:
            return self.OnSaveAs()
        self.text.SaveFile(self._file_name)
        return True
    
    def OnSaveAs(self, event=None):
        file_name=self.ShowSaveDialog()
        if file_name:
            self._file_name=file_name
            self.text.SaveFile(self._file_name)
        return False
    
    def OnPreferences(self, event):
        dialog=PrefDialog(self)
        dialog.Centre()
        result=dialog.ShowModal()
        if result == wx.ID_OK:
            self.SetEditorBackground(dialog.bg.GetColour())
            self.SetEditorForeground(dialog.fg.GetColour())
            self.SetEditorLineBackground(dialog.lbg.GetColour())
        dialog.Destroy()
    
    def OnQuit(self, event):
        self.Close()
    
    def OnClose(self, event):
        if self.CheckModified():
            self.Destroy()
    
    def OnUndo(self, event):
        if self.text.CanUndo():
            self.text.Undo()
    
    def OnRedo(self, event):
        if self.text.CanRedo():
            self.text.Redo()
    
    def OnCut(self, event):
        self.text.Cut()
    
    def OnCopy(self, event):
        self.text.Copy()
    
    def OnPaste(self, event):
        if self.text.CanPaste():
            self.text.Paste()
    
    def OnSelectAll(self, event):
        self.text.SelectAll()
    
    def OnFontBigger(self, event):
        self.SetFontSize(min(self.GetFontSize()+4,72))
        self.UpdateStatus()
    
    def OnFontSmaller(self, event):
        self.SetFontSize(max(self.GetFontSize()-4,8))
        self.UpdateStatus()
    
    def OnWordCount(self, event):
        text=self.text.GetText()
        word_count=len(text.split())
        dialog=wx.MessageDialog(self,"%d words" % word_count,"Word count",wx.ICON_INFORMATION|wx.OK)
        dialog.Centre()
        result=dialog.ShowModal()
        dialog.Destroy()
    
    def OnPageDown(self, event):
        self.text.PageDown()
    
    def OnPageUp(self, event):
        self.text.PageUp()
    
    def OnGotoLine(self, event):
        dialog=wx.NumberEntryDialog(self,"Goto Line","","Line",self.text.GetCurrentLine(),0,2**30)
        if dialog.ShowModal() == wx.ID_OK:
            line=dialog.GetValue()
            line=max(0,min(self.text.GetLineCount(),line))
            self.text.GotoLine(dialog.GetValue())
        dialog.Destroy()
    
    def OnFind(self, event):
        data=wx.FindReplaceData()
        find_dialog=wx.FindReplaceDialog(self,data,'Find')
        find_dialog.Bind(wx.EVT_FIND, self.OnDialogFind)
        find_dialog.Bind(wx.EVT_FIND_NEXT, self.OnDialogFindNext)
        find_dialog.ShowModal()
        find_dialog.Destroy()
    
    def _SplitDialogFlags(self, flags):
        down=(flags & wx.FR_DOWN) != 0
        whole_word=(flags & wx.FR_WHOLEWORD) != 0
        match_case=(flags & wx.FR_MATCHCASE) != 0
        
        flags=0
        if whole_word:
            flags |= wx.stc.STC_FIND_WHOLEWORD
        if match_case:
            flags |= wx.stc.STC_FIND_MATCHCASE
        
        return flags, down
    
    def _ShowSearchText(self, pos, text, down):
        self.text.SetCurrentPos(pos)
        self.text.EnsureCaretVisible()
        
        if down:
            self.text.SetCurrentPos(pos+len(text))
            self.text.SearchAnchor()
            self.text.SetSelection(pos,pos+len(text))
        else:
            self.text.SetCurrentPos(pos)
            self.text.SetSelection(pos,pos+len(text))
            self.text.SearchAnchor()
    
    def OnDialogFind(self, event):
        text=event.GetFindString()
        flags, down=self._SplitDialogFlags(event.GetFlags())
        
        start,end=0,self.text.GetLength()
        if not down:
            start,end=end,start # search in reverse
        pos=self.text.FindText(start,end,text,flags)
        if pos >= 0:
            self._ShowSearchText(pos, text, down)
    
    def OnDialogFindNext(self, event):
        text=event.GetFindString()
        flags, down=self._SplitDialogFlags(event.GetFlags())
        
        if down:
            pos=self.text.SearchNext(flags,text)
        else:
            pos=self.text.SearchPrev(flags,text)
        
        if pos >= 0:
            self._ShowSearchText(pos, text, down)
    
    def _ConvertAccel(self,accel):
        if wx.Platform == '__WXMAC__':
            # on mac we want to show the right symbols to match the menus
            accel=unicode(accel)
            accel=accel.replace(u'Ctrl',u'\u2318') # apple command key (clover)
            accel=accel.replace(u'Shift',u'\u21e7') # shift key (up arrow)
            accel=accel.replace(u'-DOWN',u'\u2193') # down arrow
            accel=accel.replace(u'-UP',u'\u2191') # down arrow
            return accel.replace('-','') # remove dashes
        return accel
    
    def OnList(self, event):
        # list all menu items with shortcuts
        # seeing as user can't see them normally
        max_len=0
        for menu, label in self.menubar.GetMenus():
            for item in menu.GetMenuItems():
                if item.GetAccel():
                    max_len=max(max_len,len(item.GetLabel()))
        list_text=['<pre>']
        for menu, label in self.menubar.GetMenus():
            # remove accel key shortcuts
            list_text.append(label.replace('&',''))
            list_text.append(':\n')
            for item in menu.GetMenuItems():
                accel=item.GetAccel()
                if accel:
                    list_text.append('\t')
                    list_text.append(item.GetLabel().ljust(max_len))
                    list_text.append('\t')
                    list_text.append(self._ConvertAccel(accel.ToString()))
                    list_text.append('\n')
        list_text.append('</pre>')
        list_text=u''.join(list_text)
        dialog=wx.html.HtmlHelpDialog(self,0)
        dialog.GetHelpWindow().GetHtmlWindow().SetPage(list_text)
        dialog.Centre()
        dialog.ShowModal()
        dialog.Destroy()
    
    def OnInfo(self, event):
        info=wx.AboutDialogInfo()
        info.SetName('5KEdit')
        info.SetVersion('1.0')
        info.SetDescription('A fullscreen editor in 5K of code.')
        info.AddDeveloper('John Montgomery')
        info.SetCopyright('(C) John Montgomery 2009')
        wx.AboutBox(info)
    
    def CheckModified(self):
        # check if file has been modified
        if self.text.GetModify():
            dialog=wx.MessageDialog(self,"Your changes will be lost if you don't save them.","Do you want to save your changes?",wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
            dialog.Centre()
            result=dialog.ShowModal()
            dialog.Destroy()
            if result == wx.ID_YES:
                return self.OnSave()
            elif result == wx.ID_NO:
                return True
            return False
        return True
    
    def _ShowFileDialog(self, title, flag):
        wildcard="Text Files (*.txt)|*.txt|All Files|*.*"
        dialog=wx.FileDialog(self,title,'','',wildcard, flag)
        dialog.Centre()
        path=None
        if dialog.ShowModal() == wx.ID_OK:
            path = dialog.GetPath()
        dialog.Destroy()
        return path

    def ShowSaveDialog(self):
        title="Save Text File"
        return self._ShowFileDialog(title, wx.SAVE | wx.OVERWRITE_PROMPT)

    def ShowOpenDialog(self):
        title="Open Text File"
        return self._ShowFileDialog(title, wx.OPEN)

class App(wx.App):
    
    def OnInit(self):
        self.SetAppName('5KEdit')
        frame=MainFrame(None)
        
        #frame.CenterOnScreen()
        frame.SetSize(wx.DisplaySize())
        
        frame.Show()
        frame.ShowFullScreen(True)
        frame.SetPosition((0,0))
        
        for name in sys.argv[1:]:
            frame.Load(name)
        
        return True

app=App(redirect=False)
app.MainLoop()